﻿using _123.Domain.UseCase;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace _123.UI
{
    public class PresenceConsoleUI
    {
        PresenceUseCase _presenceUseCase;
        UseCaseGeneratePresence _useCaseGeneratePresence;
        private PresenceUseCase presenceUseCase;

        public PresenceConsoleUI(PresenceUseCase presenceUseCase, UseCaseGeneratePresence useCaseGeneratePresence)
        {
            _presenceUseCase = presenceUseCase;
            _useCaseGeneratePresence = useCaseGeneratePresence;
        }

        public PresenceConsoleUI(PresenceUseCase presenceUseCase)
        {
            this.presenceUseCase = presenceUseCase;
        }

        public void GetPresenceByGroup(int groupId)
        {
            StringBuilder presenceOutput = new StringBuilder();
            var presence = _presenceUseCase.GetPresenceByGroup(groupId);
            foreach (var p in presence)
            {
                presenceOutput.AppendLine($"{p.ClassNum}\t{p.User}\t{p.Date}");
            }
            Console.WriteLine(presenceOutput);
        }

        public void GetPresenceByGroupAndDAte(int groupId, DateOnly date)
        {
            StringBuilder presenceOutput = new StringBuilder();
            var presence = _presenceUseCase.GetPresenceByGroupAndDate(groupId, date);
            foreach (var p in presence)
            {
                presenceOutput.AppendLine($"{p.ClassNum}\t{p.User}\t{p.Date}");
            }
            Console.WriteLine(presenceOutput);
        }

        public void UnCheckAttendence(int firstClass, int lastClass, DateOnly date, int userId)
        {
            var result = _presenceUseCase.UnCheckAttendance(firstClass, lastClass, date, userId);
            Console.WriteLine(result ? "Посещаемость успешно обновлена" : "Ошибка при обновлении посещаемости");
        }

        public void AddPresence(int firstClass, int lastClass, int groupId, DateOnly date)
        {
            try
            {
                _useCaseGeneratePresence.AddPrecence(firstClass, lastClass, groupId, date);
                Console.WriteLine("Посещаемость успешно добавлена");
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Ошибка при добавлении посещаемости: {ex.Message}");
            }
        }
        public void GetPresenceStatsByGroup(int groupId)
        {
            var stats = _presenceUseCase.GetPresenceStatsByGroup(groupId);
            StringBuilder output = new StringBuilder();

            output.AppendLine($"Информация о группе {groupId}:");
            output.AppendLine($"Количество студентов: {stats["Количество студентов"]}");
            output.AppendLine($"Количество занятий: {stats["Количество занятий"]}");
            output.AppendLine($"Общий процент посещаемости: {stats["Процент посещаемости"]}%");
            output.AppendLine("\nСтатистика по студентам:");

            var presence = _presenceUseCase.GetPresenceByGroup(groupId);
            var students = presence.GroupBy(p => p.User)
                                 .Select(g => new {
                                     Student = g.Key,
                                     Total = stats["Количество занятий"],
                                     Attended = g.Count(p => p.IsAttedance),
                                     Missed = stats["Количество занятий"] - g.Count(p => p.IsAttedance),
                                     Percentage = (g.Count(p => p.IsAttedance) * 100) / stats["Количество занятий"]
                                 });

            foreach (var student in students)
            {
                output.AppendLine($"\nСтудент: {student.Student.UserFIO}");
                output.AppendLine($"Посещено занятий: {student.Attended}");
                output.AppendLine($"Пропущено занятий: {student.Missed}");
                output.AppendLine($"Процент посещаемости: {student.Percentage}%");
            }

            Console.WriteLine(output.ToString());
        }
    }
       
}
