﻿using _123.Data.LocalData.Entity;
using _123.Data.ReportsHistory;
using _123.Domain.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace _123.Domain.UseCase
{
    public class UserUseCase
    {
        private UserRepositoryImpl _repositoryUserImpl;
        private GroupRepositoryImpl _repositoryGroupImpl;

        public UserUseCase(UserRepositoryImpl repositoryImpl, GroupRepositoryImpl repositoryGroupImpl)
        {
            _repositoryUserImpl = repositoryImpl;
            _repositoryGroupImpl = repositoryGroupImpl;
        }

        public List<Group> GetAllGroups() => _repositoryGroupImpl.GetAllGroups()
              .Select(it => new Group { ID = it.ID, Name = it.Name }).ToList();
        public List<User> GetAllUsers() => _repositoryUserImpl.GetAllUsers
              .Join(_repositoryGroupImpl.GetAllGroups(),
              user => user.GroupID,
              group => group.ID,
              (user, group) =>
              new User
              {
                  UserFIO = user.UserFIO,
                  UserGuid = user.UserGuid,
                  UserGroup = new Group { ID = group.ID, Name = group.Name }
              }
            ).ToList();

        public bool RemoveUserByGuid(Guid userGuid)
        {
            return _repositoryUserImpl.RemoveUserByGuid(userGuid);
        }
        public User UpdateUser(User user)
        {
            UserLocalEntity userLocalEnity = new UserLocalEntity { UserFIO = user.UserFIO, GroupID = user.UserGroup.ID, UserGuid = user.UserGuid };
            UserLocalEntity? result = _repositoryUserImpl.UpdateUser(userLocalEnity);
            if (result == null) throw new Exception("");
            Group? group = GetAllGroups().FirstOrDefault(it => it.ID == result!.GroupID);
            if (group == null) throw new Exception("");
            return new User { UserFIO = user.UserFIO, UserGuid = user.UserGuid, UserGroup = group };
        }

    }
}
